@@ -21,10 +21,8 @@
 
 #include "transcode.h"
 
-#include <algorithm>
 #include <cerrno>
 #include <cstdint>
-#include <cstring>
 #include <ctime>  // IWYU pragma: keep (time_t)
 #include <limits>
 #include <mutex>
@@ -42,7 +40,10 @@ StatsCache stats_cache;
 
 bool Transcoder::open() {
     /* Create Encoder and Decoder objects. */
-    decoder_.reset(Decoder::CreateDecoder(strrchr(filename_.c_str(), '.') + 1));
+    size_t dot_idx = filename_.rfind('.');
+    if (dot_idx != std::string::npos) {
+        decoder_ = Decoder::CreateDecoder(filename_.substr(dot_idx + 1));
+    }
     if (!decoder_) {
         errno = EIO;
         return false;
@@ -57,9 +58,7 @@ bool Transcoder::open() {
 
     Log(DEBUG) << "Decoder initialized successfully.";
 
-    stats_cache.get_filesize(filename_, decoder_->mtime(), &encoded_filesize_);
-    encoder_.reset(
-        Encoder::CreateEncoder(params.desttype, &buffer_, encoded_filesize_));
+    encoder_ = Encoder::CreateEncoder(params.desttype, &buffer_);
     if (!encoder_) {
         errno = EIO;
         return false;
@@ -78,7 +77,9 @@ bool Transcoder::open() {
     Log(DEBUG) << "Metadata processing finished.";
 
     /* Render tag from Encoder to Buffer. */
-    if (encoder_->render_tag() == -1) {
+    size_t cached_size = 0;
+    stats_cache.get_filesize(filename_, decoder_->mtime(), &cached_size);
+    if (encoder_->render_tag(cached_size) == -1) {
         Log(ERROR) << "Error rendering tag in Encoder.";
         errno = EIO;
         return false;
@@ -95,10 +96,6 @@ ssize_t Transcoder::read(char* buff, off_t offset, size_t len) {
     if (static_cast<size_t>(offset) > get_size()) {
         return 0;
     }
-    if (offset + len > get_size()) {
-        len = get_size() - offset;
-        Log(DEBUG) << "Actual length to read: " << len;
-    }
 
     // If the requested data has already been filled into the buffer, simply
     // copy it out.
@@ -107,51 +104,29 @@ ssize_t Transcoder::read(char* buff, off_t offset, size_t len) {
         return len;
     }
 
-    // If we don't already have the data and we can't produce it, return error.
-    if (!decoder_ || !encoder_) {
-        Log(ERROR) << "Can't generate data with closed coders.";
-        errno = EINVAL;
-        return -1;
+    while (decoder_ && encoder_ &&
+           buffer_.tell() < (encoder_->no_partial_encode()
+                                 ? std::numeric_limits<size_t>::max()
+                                 : offset + len)) {
+        int stat = decoder_->process_single_fr(encoder_.get());
+        if (stat == -1 || (stat == 1 && !finish())) {
+            errno = EIO;
+            return -1;
+        }
     }
 
-    if (!transcode_until(encoder_->no_partial_encode()
-                             ? std::numeric_limits<size_t>::max()
-                             : offset + len)) {
-        errno = EIO;
-        return -1;
-    }
-
-    // truncate if we didn't actually get len
-    if (buffer_.tell() < offset + len) {
-        len = std::max<off_t>(buffer_.tell() - offset, 0);
+    // truncate if we can't get len
+    size_t max_len = buffer_.max_valid_bytes(offset);
+    if (len > max_len) {
+        len = max_len;
     }
 
     buffer_.copy_into(reinterpret_cast<uint8_t*>(buff), offset, len);
 
+    Log(DEBUG) << "Successfully read " << len << " bytes.";
     return len;
 }
 
-size_t Transcoder::get_size() const {
-    if (encoded_filesize_ != 0) {
-        return encoded_filesize_;
-    }
-    if (encoder_) {
-        return encoder_->calculate_size();
-    }
-
-    return buffer_.tell();
-}
-
-bool Transcoder::transcode_until(size_t end) {
-    while (encoder_ && buffer_.tell() < end) {
-        int stat = decoder_->process_single_fr(encoder_.get());
-        if (stat == -1 || (stat == 1 && !finish())) {
-            return false;
-        }
-    }
-    return true;
-}
-
 bool Transcoder::finish() {
     // Decoder cleanup
     time_t decoded_file_mtime = 0;
@@ -167,16 +142,14 @@ bool Transcoder::finish() {
         }
 
         /* Check encoded buffer size. */
-        encoded_filesize_ = encoder_->get_actual_size();
         Log(DEBUG) << "Finishing file. Predicted size: "
                    << encoder_->calculate_size()
-                   << ", final size: " << encoded_filesize_;
+                   << ", final size: " << buffer_.size();
         encoder_.reset(nullptr);
     }
 
-    if (params.statcachesize > 0 && encoded_filesize_ != 0) {
-        stats_cache.put_filesize(filename_, encoded_filesize_,
-                                 decoded_file_mtime);
+    if (params.statcachesize > 0 && buffer_.size() != 0) {
+        stats_cache.put_filesize(filename_, buffer_.size(), decoded_file_mtime);
     }
 
     return true;
